@@ -11,8 +11,8 @@ class Defog:
         Initializes the Defog class.
         :param api_key: The API key for the defog account.
         """
-        if db_type not in ["postgres", "redshift", "mysql", "bigquery", "mongo", "snowflake"]:
-            raise Exception(f"Database `{db_type}` is not supported right now. db_type must be one of 'postgres', 'redshift', 'mysql', 'bigquery', 'mongo'")
+        if db_type not in ["postgres", "redshift", "mysql", "bigquery", "mongo", "snowflake", "sqlserver"]:
+            raise Exception(f"Database `{db_type}` is not supported right now. db_type must be one of 'postgres', 'redshift', 'mysql', 'bigquery', 'mongo', 'sqlserver'.")
         self.api_key = api_key
         self.db_type = db_type
         self.db_creds = db_creds
@@ -128,6 +128,43 @@ class Defog:
             print(resp)
             raise resp['message']
     
+    def generate_sqlserver_schema(self, tables: list):
+        try:
+            import pyodbc
+        except:
+            raise Exception("pyodbc not installed.")
+        
+        conn = pyodbc.connect(self.db_creds)
+        cur = conn.cursor()
+        schemas = {}
+
+        print("Getting schema for each table in your database...")
+        # get the schema for each table
+        for table_name in tables:
+            cur.execute(f"SELECT column_name, data_type FROM information_schema.columns WHERE table_name = '{table_name}';")
+            rows = cur.fetchall()
+            rows = [row for row in rows]
+            rows = [{"column_name": i[0], "data_type": i[1]} for i in rows]
+            schemas[table_name] = rows
+        
+        conn.close()
+
+        print("Sending the schema to the defog servers and generating a Google Sheet. This might take up to 2 minutes...")
+        # send the schemas dict to the defog servers
+        r = requests.post("https://api.defog.ai/get_postgres_schema_gsheets",
+            json={
+                "api_key": self.api_key,
+                "schemas": schemas
+            }
+        )
+        resp = r.json()
+        try:
+            gsheet_url = resp['sheet_url']
+            return gsheet_url
+        except Exception as e:
+            print(resp)
+            raise resp['message']
+
     def generate_snowflake_schema(self, tables: list):
         try:
             import snowflake.connector
@@ -260,6 +297,8 @@ class Defog:
             return self.generate_redshift_schema(tables)
         elif self.db_type == "snowflake":
             return self.generate_snowflake_schema(tables)
+        elif self.db_type == "sqlserver":
+            return self.generate_sqlserver_schema(tables)
         else:
             raise Exception("Invalid database type. Valid types are: postgres, mysql, mongo, bigquery, and redshift")
 
@@ -347,6 +386,20 @@ class Defog:
         resp = r.json()
         return resp
     
+    def update_sqlserver_schema(self, gsheet_url : str):
+        """
+        Updates the mongo schema on the defog servers.
+        :param gsheet_url: The url of the google sheet containing the schema.
+        """
+        r = requests.post("https://api.defog.ai/update_postgres_schema",
+            json={
+                "api_key": self.api_key,
+                "gsheet_url": gsheet_url
+            }
+        )
+        resp = r.json()
+        return resp
+    
     def update_db_schema(self, gsheet_url: str):
         print("Updating the schema on the Defog servers. This might take a couple minutes...")
         if self.db_type == "postgres":
@@ -361,6 +414,8 @@ class Defog:
             return self.update_redshift_schema(gsheet_url)
         elif self.db_type == "snowflake":
             return self.update_snowflake_schema(gsheet_url)
+        elif self.db_type == "sqlserver":
+            return self.update_sqlserver_schema(gsheet_url)
         else:
             raise Exception("Invalid database type. Valid types are: postgres, mysql, mongo, bigquery, and redshift")
 
@@ -575,6 +630,28 @@ class Defog:
                     }
                 except Exception as e:
                     return {"error_message": str(e), "ran_successfully": False}
+            elif query['query_db'] == "sqlserver":
+                try:
+                    import pyodbc
+                except:
+                    raise Exception("pyodbc not installed.")
+                conn = pyodbc.connect(self.db_creds)
+                cur = conn.cursor()
+                try:
+                    cur.execute(query["query_generated"])
+                    colnames = [desc[0] for desc in cur.description]
+                    result = cur.fetchall()
+                    cur.close()
+                    conn.close()
+                    print("Query ran succesfully!")
+                    return {
+                        "columns": colnames,
+                        "data": result,
+                        "query_generated": query["query_generated"],
+                        "ran_successfully": True
+                    }
+                except Exception as e:
+                    return {"error_message": str(e), "ran_successfully": False}
             else:
                 raise Exception("Database type not yet supported.")
         else:
